/*
   mem.c -- PIKT memory routines

   $Header: /home/cvs/ucgsb/pikt/src/lib/mem.c,v 1.6 2000/01/12 19:25:12 berto Exp $

   This file is part of PIKT ("Problem Informant/Killer Tool").
   Copyright (C) 1998-2000 by Robert Osterlund.

   This program is free software; you can redistribute it and/or
   modify it under the terms of the GNU General Public License
   as published by the Free Software Foundation; either version 2
   of the License, or (at your option) any later version.

   PIKT is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
   USA.

   This product includes software developed by the University of
   California, Berkeley and its contributors.

   Send problem reports, bouquets and brickbats to:

   robert.osterlund@gsb.uchicago.edu

   Robert Osterlund
   Graduate School of Business
   University of Chicago
   1101 E. 58th Street, W309
   Chicago, IL 60637
   USA
*/

#include "common.h"

/* malloc() the specified amount of memory; if unsuccessful, presumably
   because of virtual memory exhaustion, log a warning and keep trying;
   if successful, return a generic pointer to the newly allocated memory
 */
void *
mymalloc (size_t size, const char *func)
{
	void *ptr;
	static char msg[128];	/* should be plenty */

	while (! (ptr = malloc(size))) {
		if (isverboselog) {
			sprintf(msg, "in %s(), malloc() failure", func);
			if (logdir && piktc_svclog) {	/* piktc_svclog is the
			                                   last path set in
			                                   initpaths()
			                                 */
				logreturn(PIKTLOG, LOG_WARNING, __FILE__, __LINE__, FALSE, msg);
			} else {
				fprintf(stderr, msg);
				fflush(stderr);
			}
		}
		sleep(MALLOC_TIME_TO_SLEEP);
	}
	return ptr;
}

/* calloc() the specified amount of memory; if unsuccessful, presumably
   because of virtual memory exhaustion, log a warning and keep trying;
   if successful, return a generic pointer to the newly allocated memory
 */
void *
mycalloc (size_t nobj, size_t size, const char *func)
{
	void *ptr;
	static char msg[128];	/* should be plenty */

	while (! (ptr = calloc(nobj, size))) {
		if (isverboselog) {
			sprintf(msg, "in %s(), calloc() failure", func);
			if (logdir && piktc_svclog) {	/* piktc_svclog is the
			                                   last path set in
			                                   initpaths()
			                                 */
				logreturn(PIKTLOG, LOG_WARNING, __FILE__, __LINE__, FALSE, msg);
			} else {
				fprintf(stderr, msg);
				fflush(stderr);
			}
		}
		sleep(MALLOC_TIME_TO_SLEEP);
	}
	return ptr;
}

/* strdup() the string; if unsuccessful, presumably because of virtual memory
   exhaustion, log a warning and keep trying; if successful, return a char
   pointer to the newly allocated memory
 */
signed char *
mystrdup (const signed char *str, const char *func)
{
	static signed char *ptr;
	static char msg[128];	/* should be plenty */

	while (! (ptr = (signed char *) strdup(str))) {
		if (isverboselog) {
			sprintf(msg, "in %s(), strdup() failure", func);
			if (logdir && piktc_svclog) {	/* piktc_svclog is the
			                                   last path set in
			                                   initpaths()
			                                 */
				logreturn(PIKTLOG, LOG_WARNING, __FILE__, __LINE__, FALSE, msg);
			} else {
				fprintf(stderr, msg);
				fflush(stderr);
			}
		}
		sleep(MALLOC_TIME_TO_SLEEP);
	}
	return ptr;
}
